/** \file   unzcomp.c
 *  \brief  Decompression of ZDDs into hypergraphs
 *  \author Takahisa Toda
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "my_def.h"
#include "bdd_io_setfam.h"
#include "bdd_io_kf.h"

#define PROGNAME "UNZCOMP"
#define PROGVERS "1.2.2"

#ifdef SELECT_CUDD
DdManager *dd_mgr = NULL; //!< ZDD manager for CUDD
#endif /*SELECT_CUDD*/


#define PRINT_USAGE(p) do{fprintf(stderr, "Usage: %s input-file [output-file]\n", (p));} while(0)

#if defined(MISC_LOG) || defined(SIZE_LOG)
  char *g_basename; //!< basename of input file
#endif /*defined(MISC_LOG) || defined(SIZE_LOG)*/

int main(int argc, char *argv[])
{   
  int res       = ST_SUCCESS;
  char *infile  = NULL;
  char *outfile = NULL;

  /*** INPUT ARGUMENT ***/  
  for(int i = 1; i < argc; i++) {
    if(argv[i][0] == '-') {
      switch (argv[i][1]){
      case 'v': case '?': case 'h': default:
        PRINT_USAGE(argv[0]); return  ST_SUCCESS;
      }
    } else {
      if     (infile  == NULL)  {infile = argv[i];}
      else if(outfile == NULL)  {outfile = argv[i];}
      else                      {PRINT_USAGE(argv[0]); return  ST_SUCCESS;}
    }
  }
  if(infile == NULL) {PRINT_USAGE(argv[0]); return  ST_SUCCESS;}
#if defined(MISC_LOG) || defined(SIZE_LOG)
  time_t timer        = time(NULL);
  struct tm *execdate = localtime(&timer);
  g_basename          = infile;
  for(int i = 0; infile[i] != '\0'; i++) {if(infile[i] == '/') g_basename = infile+i+1;}
#endif /*defined(MISC_LOG) || defined(SIZE_LOG)*/
#ifdef MISC_LOG
  printf("date\t%s",          asctime(execdate));
  printf("program\t%s-%s\n",  PROGNAME, PROGVERS);
  printf("package\t%s\n",     BDD_PACKAGE);
  printf("input\t%s\n",       infile);
  if(outfile != NULL)         printf("output\t%s\n", outfile);
  printf("\n");
  fflush(stdout);
#endif /*MISC_LOG*/

  FILE *in = fopen(infile, "r");
  ENSURE_TRUE_MSG(in != NULL, "file open failed\n");

  /*** SETUP BDD PACKAGE ***/
  uintmax_t row_count;
  itemval maxval;
  res = getfileinfo_kf(&row_count, &maxval, in);
  ENSURE_SUCCESS(res);
  ENSURE_TRUE_MSG(maxval <= BDD_MAXITEMVAL, "the number of variable too large");
  bdd_init(maxval, 0);

  /*** IMPORT ZDD ***/  
  clock_t startingtime = current_time();
  zddp f = zdd_from_kf(row_count, in);
  ENSURE_TRUE_MSG(f != BDD_NULL, "ZDD construction failed");
  print_elapsed_time(startingtime, "INPUT");
#ifdef MISC_LOG
  printf("|zdd|\t%ju\n",      zdd_size(f));
  printf("#sets\t%ju\n",      zdd_card(f));
  printf("\n");
  fflush(stdout);
#endif /*MISC_LOG*/

  /*** EXPORT ZDD ***/  
  if(outfile != NULL) {
    FILE *out     = fopen(outfile, "w");
    startingtime  = current_time();
    unzcomp(maxval, f, out);
    print_elapsed_time(startingtime, "OUTPUT");
    fclose(out);
  }

  bdd_quit();
  fclose(in);

  return ST_SUCCESS;
}


